local paramsutil = require "paramsutil"
local polygonutil = require "polygonutil"
local transf = require "transf"
local tableutil = require "tableutil"
local vec3 = require "vec3"

local manio_underground_station = { }

function manio_underground_station.makeParams()
	params = {}
	params[#params+1] = {
		key = "tracksLength",
		name = _("Length of tracks"),
		values = { _("80m"), _("160m"), _("240m"), _("320m") },
		defaultIndex = 1
	}
	params[#params+1] = {
		key = "numTracks",
		name = _("Number of tracks"),
		values = { _("1"), _("2"), _("3"), _("4"), _("5"), _("6"), _("7"), _("8") },
		defaultIndex = 0
	}
	params[#params+1] =	paramsutil.makeTrackTypeParam()
	params[#params+1] =	paramsutil.makeTrackCatenaryParam()
	return params
end

function manio_underground_station.makeUndergroundTrainStation(params, year)
	local tracksConfig = {
		trackTypes = { "standard.lua", "high_speed.lua" },
	}
	local lengths = { 80, 160, 240, 320}
	local depth = -18;
	local result = { }
	result.models = { }
	result.edgeLists = { }
	result.terminalGroups = { }
	result.groundFaces = { }

	local trackDistance = params.state.track.trackDistance;
	local trackType = tracksConfig.trackTypes[(params.trackType or 0) + 1]
	local numTracks = params.numTracks;
	local idxLength = (params.tracksLength or 0);
	local conLength = lengths[(params.tracksLength or 0) + 1];
	local myear = year or "1850";
	
	-- ##### Train tracks #####
	local edgesT = { }
	local snapNodesT = { }
	for track = 0, numTracks, 1 do
		local to = track*trackDistance/2;
		if (track%2==0) then
			to = (-1*to) - (trackDistance/2);
		end
		to = to + trackDistance/2;
		edgesT[#edgesT + 1] = { { to,	-conLength/2,	depth	},	{ 0,	conLength/4,	0 } }
		edgesT[#edgesT + 1] = { { to,	0,				depth	},	{ 0,	conLength/4,	0 } }
		edgesT[#edgesT + 1] = { { to,	0,				depth	},	{ 0,	conLength/4,	0 } }
		edgesT[#edgesT + 1] = { { to,	conLength/2,	depth	},	{ 0,	conLength/4,	0 } }
		snapNodesT[#snapNodesT+1] = #edgesT-4
		snapNodesT[#snapNodesT+1] = #edgesT-1
	end
	result.edgeLists[#result.edgeLists + 1] = {
		type = "TRACK",
		edgeType = "TUNNEL",
		edgeTypeName = "railroad_old.lua",
		params = {
			type = trackType,
			catenary = params.catenary == 1
		},
		edges = edgesT,
		snapNodes = snapNodesT,
		--alignTerrain = true
	}

	-- ##### Street #####
	local roadEdges = { }
	local roadSnapNodes = { }
	roadEdges[#roadEdges +1] = { {-20, 0, .0}, {20, 0, .0} }
	roadSnapNodes[#roadSnapNodes +1] = #roadEdges-1
	roadEdges[#roadEdges +1] = { {0, 0, .0}, {20, 0, .0} }
	result.edgeLists[#result.edgeLists + 1] = {
		type = "STREET",
		params = { type = "new_medium.lua", tramTrackType = "ELECTRIC" },
		edges = roadEdges,
		snapNodes = roadSnapNodes
	}

	local terminals = { }

	-- ##### Main building #####
	result.models[#result.models + 1] = {
		id = "manio_uts001/"..myear.."/passenger_mainBuilding.mdl",
		transf = transf.rotZYXTransl(transf.degToRad(270.0, 0.0, 0.0), vec3.new(15, 0, 0)),
	}
	result.terminalGroups[#result.terminalGroups + 1] = { terminals = {{ #result.models - 1, 0 }} }
	-- ##### Courtyard #####
	local dcy = 22;
	if (myear == "1850") then
		dcy = 21;
	end
	result.models[#result.models + 1] = {
		id = "manio_uts001/courtyard.mdl",
		transf = transf.rotZYXTransl(transf.degToRad(180.0, 0.0, 0.0), vec3.new(dcy, -10, 0)),
	}
	result.models[#result.models + 1] = {
		id = "manio_uts001/courtyard.mdl",
		transf = transf.rotZYXTransl(transf.degToRad(0.0, 0.0, 0.0), vec3.new(dcy, 10, 0)),
	}
	-- ##### Underground platforms #####
	for track = 0, numTracks, 1 do
		terminals = { }
		local dx = track*4
		if (track%2==0) then
			dx = -1*dx - 4;
		end
		dx = 11 + dx;
		local numSegments = conLength / 20 / 2;
		for seg=-numSegments, numSegments-1, 1 do
			result.models[#result.models + 1] = {
				id = "manio_uts001/underground_platform_segment.mdl",
				transf = transf.rotZYXTransl(transf.degToRad(90.0, 0.0, 0.0), vec3.new(dx, (seg*20)+10, 0)),
			}
			terminals[#terminals + 1] = { #result.models - 1, 0 }
		end
		result.terminalGroups[#result.terminalGroups + 1] = { terminals = terminals, vehicleNodeOverride = track*4 + 2 }
	end

	-- ##### Floor etc #####
	local faces = { { -0.1, -21.3, .0}, { 25, -21.3, .0}, { 25, 21.3, .0}, { 0.1, 21.3, .0} }
	result.terrainAlignmentLists = {{ type = "EQUAL", faces = { faces } }}
	result.groundFaces[#result.groundFaces + 1] = { face = faces, modes = { { type = "FILL", key = "industry_concrete_01" } } }
	result.groundFaces[#result.groundFaces + 1] = { face = faces, modes = { { type = "STROKE_OUTER", key = "building_paving" } } }
	result.cost = 60000 + numTracks * 24000
	result.maintenanceCost = result.cost / 6
	return result
end
	
return manio_underground_station